// Copyright (C) 2013 - 2016 - Michael Baudin
// Copyright (C) 2010 - DIGITEO - Michael Baudin
// Copyright (C) 1993 - 1995 - Anders Holtsberg
//
// This file must be used under the terms of the CeCILL.
// This source file is licensed as described in the file COPYING, which
// you should have received as part of this distribution.  The terms
// are also available at
// http://www.cecill.info/licences/Licence_CeCILL_V2-en.txt

function [p,S,mu] = polyfit(x,y,n)
    // Polynomial curve fitting
    //
    // Calling Sequence
    //   p = polyfit(x,y,n)
    //   [p,S] = polyfit(x,y,n)
    //   [p,S,mu] = polyfit(x,y,n)
    //
    // Parameters
    //   x : a nx-by-mx matrix of doubles
    //   y : a nx-by-mx matrix of doubles
    //   n : a 1-by-1 matrix of doubles, integer value, n>=0
    //   p : a 1-by-(n+1) matrix of doubles, the coefficients of the polynomial
    //   S : a structure to estimate the errors of evaluations (see below )
    //   mu : a 1-by-2 matrix of doubles, the standardizing parameter : mu(1) is mean(x) and mu(2)=stdev(x). Use mu to standardize the data for a more accurate evaluation of p, using the mean and the standard deviation. 
    //
    // Description
    // <literal>polyfit(x,y,n)</literal> finds the coefficients of a polynomial p(x) of
    // degree n that fits the data, p(x(i)) to y(i), in a least-squares sense.
    //
    // These coefficients are ordered with powers in decreasing order:
    //
    // <latex>
    // p(x) = p_1 x^n + p_2 x^{n-1} + ... + p_n x + p_{n+1}.
    // </latex>
    //
    // The implementation is based on the QR decomposition. 
    // 
    // <literal>[p,S]=polyfit(x,y,n)</literal> returns the polynomial coefficients 
    // p and a data structure S for use with <literal>polyval</literal> to produce 
    // error estimates on predictions.
    //
    // The following are the fields of S.
    // <itemizedlist>
    //   <listitem><para>
    // S.R : Cholesky factors from the QR decomposition
    //   </para></listitem>
    //   <listitem><para>
    // S.df : the number of degrees of freedom : nx-(n+1)
    //   </para></listitem>
    //   <listitem><para>
    // S.norm : the 2-norm of the residuals
    //   </para></listitem>
    // </itemizedlist>
    //
    // The computation of the coefficients p can be difficult in the case where 
    // the inputs x are of very different orders of magnitude. 
    // In order to get more accurate results, we can use the mu output argument. 
    // This has the effect of computing the standard variable 
    // z from :
    //
    // <latex>
    // z_i = \frac{x_i-\overline{x}}{\sigma_x}
    // </latex>
    //
    // where <latex>\overline{x}</latex> is the mean and 
    // <latex>\sigma_x</latex> is the standard deviation. 
    // This reduces the condition number of the associated Vandermonde 
    // matrix. 
    // In this case, the mu argument must be passed to the polyval function 
    // in order to evaluate the polynomial. 
    // This allows polyval to evaluate the polynomial depending on z. 
    //
    // Examples
    // x = linspace(0,%pi,10)';
    // y = sin(x)
    // p = polyfit(x,y,3)
    // // Evaluate :
    // f = polyval(p,x)
    // // Compare with a table
    // disp([x y f abs(f-y)])
    // // Compare with a polynomial
    // ff = p(1)*x.^3+p(2)*x.^2+p(3)*x+p(4)
    // // Compare with a graphics
    // // Notice that the fit is good up to x=%pi, 
    // // then the fit is not good anymore.
    // x = linspace(0,4,100)';
    // y = sin(x);
    // f = polyval(p,x);
    // scf();
    // plot(x,y,"r-")
    // plot(x,f,"b-")
    // plot([%pi %pi],[-1 1],"g-")
    // legend(["Sin","Polynomial","Limit"],"in_lower_left");
    // xtitle("Degree 3 least squares polynomial","X","F(x)")
    //
    // // A difficult example : scaling is necessary.
    // x=[-1e15 1.e6 1e15]; 
    // y=[-12. 0 12.]; 
    // p=polyfit(x,y,2) // A warning is generated
    // [p,S,mu]=polyfit(x,y,2) // No problem
    // // Evaluate :
    // y2=polyval(p,x,S,mu)
    //
    // // Source [2]
    // // Caution : the urban definition changes in 1950
    // cdate=(1790:10:1990)';
    // pop=[
    //     3.929214   
    //     5.308483   
    //    7.239881   
    //    9.638453   
    //    12.860702  
    //    17.063353  
    //    23.191876  
    //    31.443321  
    //    38.558371  
    //    50.189209  
    //    62.979766  
    //    76.212168  
    //    92.228496  
    //    106.02154  
    //    123.20262  
    //    132.16457  
    //    151.3258   
    //    179.32317  
    //    203.30203  
    //    226.5422   
    //    248.70987  
    // ];
    // scf();
    // // Plot the data
    // plot(cdate,pop,"+")
    // // Calculate fit parameters
    // [p,S] = polyfit(cdate,pop,2);
    // // Evaluate the fit
    // pop_fit = polyval(p,cdate,S);
    // // Plot the fit
    // plot(cdate,pop_fit,"g-")
    // // Annotate the plot
    // legend("Polynomial Model","Data","Location","in_upper_left");
    // xtitle("","Census Year","Population (millions)");
    // 
    // Authors
    // Copyright (C) 2013 - 2016 - Michael Baudin
    // Copyright (C) 2010 - DIGITEO - Michael Baudin
    // Copyright (C) 1993 - 1995 - Anders Holtsberg
    //
    // Bibliography
    // [1] http://en.wikipedia.org/wiki/Polynomial_interpolation
    // [2] USA population : Population: 1790 to 1990, http://www.census.gov/population/censusdata/table-4.pdf
    // [3] http://www.mathworks.fr/fr/help/matlab/data_analysis/programmatic-fitting.html
    // [4] Numerical Computing with MATLAB, Cleve Moler, 2004
    // 

    [lhs,rhs]=argn()
    apifun_checkrhs ( "polyfit" , rhs , 3:3 )
    apifun_checklhs ( "polyfit" , lhs , 1:3 )
    //
    // Check Type
    apifun_checktype ( "polyfit" , x , "x" , 1 , "constant" )
    apifun_checktype ( "polyfit" , y , "y" , 2 , "constant" )
    apifun_checktype ( "polyfit" , n , "n" , 3 , "constant" )
    //
    // Check Size
    nx=size(x,"*")
    apifun_checkvector ( "polyfit" , x , "x" , 1 , nx )
    apifun_checkvector ( "polyfit" , y , "y" , 2 , nx )
    apifun_checkscalar ( "polyfit" , n , "n" , 3 )
    //
    // Check content
    apifun_checkgreq ( "polyfit" , n , "n" , 3 , 0 )
    apifun_checkflint ( "polyfit" , n , "n" , 3 )
    //
    x=x(:)
    y=y(:)
    //
    if lhs==3 then
        // Compute mean and standard deviation
        mu(1)=mean(x)
        mu(2)=stdev(x)
        // Standardize the input x
        x=(x-mu(1))/mu(2)
    end
    //
    //
    vander=makematrix_vandermonde(x,n+1)
    // Reverse the columns
    vander=vander(:,$:-1:1)
    [Q,R] = qr(vander,"e");
    Q = Q(:,1:size(R,2))
    R = R(1:size(R,2),:)
    //
    condR=cond(R)
    if (condR>1/sqrt(%eps)) then
        warning(msprintf("%s: The matrix R is ill-conditionned : %e.\n Please use the standardizing mu output argument.\n","polyfit",condR));
    end
    //
    p = R\(Q'*y); 
    r = y - vander*p; //residuals
    p=p';
    freed = nx - (n+1); //degree of freedom

    //on return : Choleski factor and the norm of residuals
    S.R=R
    S.df=freed
    S.normr=norm(r)
endfunction
